@@ -352,8 +352,7 @@ class OptunaParam(NamedTuple):
         if self.dtype == "categorical":
             return method(self.name, self.values)
         low, high = self.values
-        config = {} if self.config is None else self.config
-        return method(self.name, low, high, **config)
+        return method(self.name, low, high, **(self.config or {}))
 
 
 optuna_params_type = Dict[str, Union[OptunaParam, Dict[str, Any], str]]
@@ -719,7 +718,7 @@ class OptunaKeyMapping(LoggingMixin):
             for k, v in d.items():
                 usage, user_prefix = self.converter.get_usage(k)
                 if usage is not None:
-                    current[k] = self.converter.parse(usage, v)
+                    current[usage] = self.converter.parse(usage, v)
                     continue
                 if isinstance(v, dict):
                     _inject_values(v, current.setdefault(k, {}))
@@ -832,13 +831,18 @@ class OptunaPresetParams:
     def _linear_preset(self) -> optuna_params_type:
         return shallow_copy_dict(self.base_params)
 
+    @staticmethod
+    def _get_head_config(params: Dict[str, Any], pipe: str) -> Dict[str, Any]:
+        model_config = params.setdefault("model_config", {})
+        pipe_configs = model_config.setdefault("pipe_configs", {})
+        return pipe_configs.setdefault(pipe, {}).setdefault("head", {})
+
     def _fcnn_preset(self) -> optuna_params_type:
         params = shallow_copy_dict(self.base_params)
         if self.kwargs.get("tune_hidden_units", True):
             hu_param = OptunaParamConverter.make_hidden_units("", 8, 2048, 3)
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            model_config.update(hu_param)
+            head_config = self._get_head_config(params, "fcnn")
+            head_config.update(hu_param)
         mapping_config: optuna_params_type = {}
         if self.kwargs.get("tune_batch_norm", True):
             bn_param = OptunaParam("mlp_batch_norm", [False, True], "categorical")
@@ -848,9 +852,8 @@ class OptunaPresetParams:
         if self.kwargs.get("tune_pruner", True):
             mapping_config.update(OptunaParamConverter.make_pruner_config(""))
         if mapping_config:
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            model_config["mapping_configs"] = mapping_config
+            head_config = self._get_head_config(params, "fcnn")
+            head_config["mapping_configs"] = mapping_config
         if self.kwargs.get("tune_embedding_dim", True):
             ed_param = OptunaParam("embedding_dim", [8, "auto"], "categorical")
             model_config = params.setdefault("model_config", {})
@@ -861,37 +864,32 @@ class OptunaPresetParams:
     def _tree_dnn_preset(self) -> optuna_params_type:
         params = self._fcnn_preset()
         if self.kwargs.get("tune_dndf", True):
-            dndf_param = OptunaParamConverter.make_dndf_config("", 64, 6, False)
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            model_config.update(dndf_param)
+            dndf_param = OptunaParamConverter.make_dndf_config("", 32, 4, False)
+            head_config = self._get_head_config(params, "dndf")
+            head_config.update(dndf_param)
         return params
 
     def _tree_linear_preset(self) -> optuna_params_type:
         params = shallow_copy_dict(self.base_params)
         if self.kwargs.get("tune_dndf", True):
-            dndf_param = OptunaParamConverter.make_dndf_config("", 64, 6, True)
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            model_config.update(dndf_param)
+            dndf_param = OptunaParamConverter.make_dndf_config("out", 64, 4, True)
+            head_config = self._get_head_config(params, "tree_stack")
+            head_config.update(dndf_param)
         return params
 
     def _tree_stack_preset(self) -> optuna_params_type:
         params = shallow_copy_dict(self.base_params)
         if self.kwargs.get("tune_num_blocks", True):
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            model_config["num_blocks"] = OptunaParam("num_blocks", [0, 3], "int")
-        if self.kwargs.get("tune_inner_dndf", False):
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            inner_param = OptunaParamConverter.make_dndf_config("", 64, 6, True)
-            model_config.update(inner_param)
-        if self.kwargs.get("tune_dndf", False):
-            model_config = params.setdefault("model_config", {})
-            assert isinstance(model_config, dict)
-            out_param = OptunaParamConverter.make_dndf_config("out", 64, 6, True)
-            model_config.update(out_param)
+            head_config = self._get_head_config(params, "tree_stack")
+            head_config["num_blocks"] = OptunaParam("num_blocks", [1, 3], "int")
+        if self.kwargs.get("tune_inner_dndf", True):
+            head_config = self._get_head_config(params, "tree_stack")
+            inner_param = OptunaParamConverter.make_dndf_config("", 64, 4, True)
+            head_config.update(inner_param)
+        if self.kwargs.get("tune_dndf", True):
+            head_config = self._get_head_config(params, "tree_stack")
+            out_param = OptunaParamConverter.make_dndf_config("out", 32, 4, True)
+            head_config.update(out_param)
         return params
 
     # TODO : optimize these three preset
