@@ -2,17 +2,25 @@ import os
 import json
 import torch
 
+import numpy as np
+
+from abc import ABCMeta
 from typing import Any
 from typing import Dict
 from typing import List
+from typing import Type
 from typing import Union
+from typing import Callable
 from typing import Optional
 from functools import partial
 from cftool.ml import ModelPattern
+from cftool.misc import register_core
 from cftool.misc import shallow_copy_dict
 from cftool.misc import lock_manager
 from cftool.misc import Saving
 from cftool.misc import LoggingMixin
+from mlflow.pyfunc import PythonModel
+from mlflow.pyfunc import PythonModelContext
 
 from ..types import data_type
 from ..types import np_dict_type
@@ -159,9 +167,7 @@ class Pack(LoggingMixin):
                 trainer = pipeline.trainer
                 inference = trainer.inference
                 if inference.binary_threshold is None:
-                    loader = trainer.binary_threshold_loader
-                    loader_name = trainer.binary_threshold_loader_name
-                    inference.generate_binary_threshold(loader, loader_name)
+                    trainer._generate_binary_threshold()
                 json.dump(inference.binary_config, f)
             if compress:
                 Saving.compress(abs_folder, remove_original=remove_original)
@@ -227,4 +233,47 @@ class Pack(LoggingMixin):
         return predictor
 
 
-__all__ = ["Pack"]
+python_models: Dict[str, Type["PythonModelBase"]] = {}
+
+
+class PythonModelBase(PythonModel, metaclass=ABCMeta):
+    """ Should implement self.predictor """
+
+    def predict(
+        self,
+        context: PythonModelContext,
+        model_input: Any,
+    ) -> np.ndarray:
+        """
+        model_inputs should be a pd.DataFrame which looks like:
+
+        |    file    |   other keys   |
+        |  "xxx.csv" |  other values  |
+
+        """
+        columns = model_input.columns
+        data = model_input.values.ravel()
+        kwargs = dict(zip(columns[1:], data[1:]))
+        return self.predictor.predict(data[0], **kwargs)
+
+    @classmethod
+    def register(cls, name: str) -> Callable[[Type], Type]:
+        global python_models
+        return register_core(name, python_models)
+
+
+@PythonModelBase.register("pack")
+class PackModel(PythonModelBase):
+    def load_context(self, context: PythonModelContext) -> None:
+        pack_folder = context.artifacts["pack_folder"]
+        self.predictor = Pack.get_predictor(pack_folder, compress=False)
+
+
+@PythonModelBase.register("pipeline")
+class PipelineModel(PythonModelBase):
+    def load_context(self, context: PythonModelContext) -> None:
+        export_folder = context.artifacts["export_folder"]
+        self.predictor = Pipeline.load(export_folder, compress=False)
+
+
+__all__ = ["Pack", "PackModel", "PipelineModel"]
